﻿//----------------------------------------------------------------------
//
// Copyright (c) Microsoft Corporation.
// All rights reserved.
//
// This code is licensed under the MIT License.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files(the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions :
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//
//------------------------------------------------------------------------------


using Microsoft.Identity.Core.Http;
using System.Net.Http.Headers;

namespace Microsoft.Identity.Core
{
    internal class ExceptionDetail
    {
        /// <summary>
        /// Gets the status code returned from http layer. This status code is either the HttpStatusCode in the inner
        /// HttpRequestException response or
        /// NavigateError Event Status Code in browser based flow (See
        /// http://msdn.microsoft.com/en-us/library/bb268233(v=vs.85).aspx).
        /// You can use this code for purposes such as implementing retry logic or error investigation.
        /// </summary>
        public int StatusCode { get; set; } = 0;

        /// <summary>
        /// The specific error codes that may be returned by the service.
        /// </summary>
        public string[] ServiceErrorCodes { get; set; }

        public string Claims { get; set; }

        public IHttpWebResponse Response { get; set; }

        /// <summary>
        /// Raw response body received from the server.
        /// </summary>
        public string ResponseBody { get; set; }

        public HttpResponseHeaders HttpResponseHeaders { get; set; }

        public static ExceptionDetail FromHttpResponse(IHttpWebResponse response)
        {
            return new ExceptionDetail()
            {
                ResponseBody = response?.Body,
                StatusCode = response != null ? (int)response.StatusCode : -1,
                HttpResponseHeaders = response?.Headers,
                Response = response
            };
        }
    }
}
